package xenon2k4;

import jge2d.Ponto;
import jge2d.TileSet;

/**
 * <p><b>Title:</b> Xenon2k4</p>
 * <p><b>Description:</b> Classe abstracta que representa todos os prmios que oferecem componentes  Nave.</p>
 * <p><b>Copyright:</b> IPL/ESTG(c)2004</p>
 * <p><b>Company:</b> 4thFloor</p>
 * @author Ruben Pedro
 * @author Srgio Lopes
 * @version 1.1
 */

public abstract class AddOn extends Premio {

  private Componente componente;


/////////////////// Constructors ///////////////////////////////////////////////

  /** Construtor
   * 
	 * @param nome Nome do prmio
	 * @param imagens TileSets do prmio AddOn
	 * @param posicao Posio na qual o prmio AddOn vai ser colocado
	 * @param asa Indica a asa (lado) na qual o componente vai ser colocado
	 * @param tempoActual Instante em que se inicia a animao e se coloca o prmio AddOn no jogo
	 * @param nivel Nivel ao qual o prmio AddOn pertence
	 * @param nomeAnimacao Nome da animao
	 * @param animacoes Sequncia de animaes a serem apresentadas
	 * @param tempos Durao de cada sequncia de animao (por Tile)
	 * @param opcaoAni Indica se deve terminar a animao actual ou no
	 */
  public AddOn(String nome, TileSet imagens, Ponto posicao, int asa, long tempoActual, Nivel nivel, String nomeAnimacao, int[][] animacoes, int[] tempos, boolean opcaoAni) {
    super(nome, imagens, posicao, tempoActual, nivel, nomeAnimacao, animacoes, tempos, opcaoAni);
  }


/////////////////// Getters & Setters //////////////////////////////////////////
	
  /**
   * @return Devolve o componente que o prmio AddOn possui
   */
	public Componente getComponente() {
		return componente;
	}	

	/**
	 * @param componente Componente a atribuir ao prmio AddOn
	 */
	public void setComponente(Componente componente) {
		this.componente = componente;
	}


/////////////////// Mtodos de Trata Coliso ///////////////////////////////////

  /* (non-Javadoc)
   * @see xenon2k4.Premio#trataColisao(xenon2k4.Componente, long)
   */
  public void trataColisao(Componente componente, long tempoActual) {
  	trataColisao(componente.getNave(), tempoActual);
    morrer(tempoActual);
  }

  /* (non-Javadoc)
	 * @see xenon2k4.Premio#trataColisao(xenon2k4.Nave, long)
	 */
	public void trataColisao(Nave nave, long tempoActual) {
		if (nave.getComponentes().size() < 4 && !nave.existeComponente(componente.getComponentType(), componente.getLado())) {
			componente.setNave(nave);
			nave.getComponentes().add(componente);
			componente.getMovimento().iniciaMovimento(tempoActual, componente.sprite);
			nivel.mapa.addSprite(componente.sprite);
		}
		morrer(tempoActual);
  }

}
