package xenon2k4;

import jge2d.Ponto;
import jge2d.TileSet;
import jge2d.jogo.movimentos.Movimento;

/**
 * <p><b>Title:</b> Xenon2k4</p>
 * <p><b>Description:</b> Entidade abstracta que representa todos os objectos que possuem uma energia.</p>
 * <p><b>Copyright:</b> IPL/ESTG(c)2004</p>
 * <p><b>Company:</b> 4thFloor</p>
 * @author Ruben Pedro
 * @author Srgio Lopes
 * @version 1.1
 */

public abstract class Energetico extends Destruidor {

	private int  energia;


/////////////////// Constructors ///////////////////////////////////////////////

	public Energetico() {
	}

	/** Construtor
	 * 
   * @param nome Nome do objecto energtico
   * @param imagens TileSets do objecto energtico
   * @param posicao Posio na qual o objecto energtico vai ser colocado
   * @param movimento Tipo de movimento a associar ao objecto energtico
	 * @param energia Quantidade de energia que o objecto energtico contm
   * @param hitPoints Capacidade de destruio do objecto energtico
   * @param tempoActual Instante em que se inicia a animao e se coloca o objecto energtico no jogo
   * @param nivel Nivel ao qual o objecto energtico pertence
   * @param nomeAnimacao Nome da animao
   * @param animacoes Sequncia de animaes a serem apresentadas
   * @param tempos Durao de cada sequncia de animao (por Tile)
   * @param opcaoAni Indica se deve terminar a animao actual ou no
   */
	public Energetico(String nome, TileSet imagens, Ponto posicao, Movimento movimento, int energia, int hitPoints, long tempoActual, Nivel nivel, String nomeAnimacao, int[][] animacoes, int[] tempos, boolean opcaoAni) {
		super(nome, imagens, posicao, movimento, hitPoints, tempoActual, nivel, nomeAnimacao, animacoes, tempos, opcaoAni);
		this.energia = energia;
	}
	

/////////////////// Getters & Setters //////////////////////////////////////////

  /**
   * @return Devolve a quantidade de energia que o objecto energtico contm
   */
	public int getEnergia() {
	  return energia;
 	}

 	/**
	 * @param energia Quantidade de energia a atribuir ao objecto energtico
	 */
	public void setEnergia(int energia) {
  	this.energia = energia;
 	}


/////////////////// Mtodos Gerais /////////////////////////////////////////////
 
  /** Retira uma certa quantidade de energia ao objecto energtico
   * 
   * @param quantidade Quantidade de energia a retiriar
   */
 	public void retirarEnergia(int quantidade){
 		energia -= quantidade;
 	}
 
 	/** Verifica se o objecto energtico ainda contm energia
 	 * 
	 * @return Verdadeiro se tiver energia, falso caso contrrio
	 */
	public boolean temEnergia(){
 		return energia > 0; 
 	}

}
