package xenon2k4;

import jge2d.Ponto;
import jge2d.TileSet;

/**
 * <p><b>Title:</b> Xenon2k4</p>
 * <p><b>Description:</b> Classe abstracta que representa todos os prmios que oferecem tiros a uma Nave ou a um Componente.</p>
 * <p><b>Copyright:</b> IPL/ESTG(c)2004</p>
 * <p><b>Company:</b> 4thFloor</p>
 * @author Ruben Pedro
 * @author Srgio Lopes
 * @version 1.1
 */

public abstract class PowerUp extends Premio {

  private TiroMachine tiro;
  

/////////////////// Constructors ///////////////////////////////////////////////

  /** Construtor
   * 
	 * @param nome Nome do prmio
	 * @param imagens TileSets do prmio
	 * @param posicao Posio na qual o prmio vai ser colocado
	 * @param tempoActual Instante em que se inicia a animao e se coloca o prmio no jogo
	 * @param nivel Nivel ao qual o prmio pertence
	 * @param nomeAnimacao Nome da animao
	 * @param animacoes Sequncia de animaes a serem apresentadas
	 * @param tempos Durao de cada sequncia de animao (por Tile)
	 * @param opcaoAni Indica se deve terminar a animao actual ou no
	 */
  public PowerUp(String nome, TileSet imagens, Ponto posicao, long tempoActual, Nivel nivel, String nomeAnimacao, int[][] animacoes, int[] tempos, boolean opcaoAni) {
    super(nome, imagens, posicao, tempoActual, nivel, nomeAnimacao, animacoes, tempos, opcaoAni);
  }


/////////////////// Getters & Setters //////////////////////////////////////////
	
  /**
   * @param novoTiro Tiro a atribuir ao prmio AddOn
   */
  public void setTiro(TiroMachine novoTiro) {
    tiro = novoTiro;
  }


/////////////////// Mtodos de Trata Coliso ///////////////////////////////////

  /** Trata a coliso de uma Machine com um PowerUp 
   * 
   * @param machine Nave ou Componente com o qual o prmio colidiu
   * @param tempoActual Instante em que colidiu
   */
  public void trataColisao(Machine machine, long tempoActual) {
    machine.setArma(tiro);
    morrer(tempoActual);
  }

}
